#!/usr/bin/env python

from distutils.core import setup

setup(name='injections',
      version='0.2.1',
      description='Simple dependency injection library',
      author='Paul Colomiets',
      author_email='paul@colomiets.name',
      url='http://github.com/tailhook/injections',
      packages=[
        'injections',
        ],
      classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 2',
        ],
     )
